@@ -12,6 +12,30 @@
 using namespace Hyprgraphics;
 using namespace Hyprutils::Memory;
 
+Hyprgraphics::CImage::CImage(const std::span<uint8_t>& data, eImageFormat format) {
+    std::expected<cairo_surface_t*, std::string> CAIROSURFACE;
+    if (format == eImageFormat::IMAGE_FORMAT_PNG) {
+        CAIROSURFACE = PNG::createSurfaceFromPNG(data);
+        mime         = "image/png";
+    } else {
+        lastError = "Currently only PNG images are supported for embedding";
+        return;
+    }
+
+    if (!CAIROSURFACE) {
+        lastError = CAIROSURFACE.error();
+        return;
+    }
+
+    if (const auto STATUS = cairo_surface_status(*CAIROSURFACE); STATUS != CAIRO_STATUS_SUCCESS) {
+        lastError = std::format("Could not create surface: {}", cairo_status_to_string(STATUS));
+        return;
+    }
+
+    loadSuccess   = true;
+    pCairoSurface = makeShared<CCairoSurface>(CAIROSURFACE.value());
+}
+
 Hyprgraphics::CImage::CImage(const std::string& path) : filepath(path) {
     std::expected<cairo_surface_t*, std::string> CAIROSURFACE;
     const auto                                   len = path.length();
@@ -45,15 +69,23 @@ Hyprgraphics::CImage::CImage(const std::string& path) : filepath(path) {
         magic_load(handle, nullptr);
 
         const auto type_str   = std::string(magic_file(handle, path.c_str()));
-        const auto first_word = type_str.substr(0, type_str.find(" "));
+        const auto first_word = type_str.substr(0, type_str.find(' '));
 
         if (first_word == "PNG") {
             CAIROSURFACE = PNG::createSurfaceFromPNG(path);
             mime         = "image/png";
-        } else if (first_word == "JPEG") {
+        } else if (first_word == "JPEG" && !type_str.contains("XL") && !type_str.contains("2000")) {
             CAIROSURFACE  = JPEG::createSurfaceFromJPEG(path);
             imageHasAlpha = false;
             mime          = "image/jpeg";
+        } else if (first_word == "JPEG" && type_str.contains("XL")) {
+#ifdef JXL_FOUND
+            CAIROSURFACE = JXL::createSurfaceFromJXL(path);
+            mime         = "image/jxl";
+#else
+            lastError = "hyprgraphics compiled without JXL support";
+            return;
+#endif
         } else if (first_word == "BMP") {
             CAIROSURFACE  = BMP::createSurfaceFromBMP(path);
             imageHasAlpha = false;
